/*
                                     CDFLIB

            Library of Fortran Routines for Cumulative Distribution
                 Functions, Inverses, and Other Parameters

                               (June, 1993)

                            Compiled and Written by:

                                 Barry W. Brown
                                  James Lovato
                                   
                     Department of Biomathematics, Box 237
                     The University of Texas, M.D. Anderson Cancer Center
                     1515 Holcombe Boulevard
                     Houston, TX      77030

 This work was supported by grant CA-16672 from the National Cancer Institute.

Code that appeared  in an    ACM  publication  is subject  to    their
algorithms policy:

     Submittal of  an  algorithm    for publication  in   one of   the  ACM
     Transactions implies that unrestricted use  of the algorithm within  a
     computer is permissible.   General permission  to copy and  distribute
     the algorithm without fee is granted provided that the copies  are not
     made  or   distributed for  direct   commercial  advantage.    The ACM
     copyright notice and the title of the publication and its date appear,
     and  notice is given that copying  is by permission of the Association
     for Computing Machinery.  To copy otherwise, or to republish, requires
     a fee and/or specific permission.

     Krogh, F.  Algorithms  Policy.  ACM  Tran.   Math.  Softw.   13(1987),
     183-186.

We place the CDFLIB code that we have written in the public domain.  

                                 NO WARRANTY
     
     WE PROVIDE ABSOLUTELY  NO WARRANTY  OF ANY  KIND  EITHER  EXPRESSED OR
     IMPLIED,  INCLUDING BUT   NOT LIMITED TO,  THE  IMPLIED  WARRANTIES OF
     MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK
     AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS  WITH YOU.  SHOULD
     THIS PROGRAM PROVE  DEFECTIVE, YOU ASSUME  THE COST  OF  ALL NECESSARY
     SERVICING, REPAIR OR CORRECTION.
     
     IN NO  EVENT  SHALL THE UNIVERSITY  OF TEXAS OR  ANY  OF ITS COMPONENT
     INSTITUTIONS INCLUDING M. D.   ANDERSON HOSPITAL BE LIABLE  TO YOU FOR
     DAMAGES, INCLUDING ANY  LOST PROFITS, LOST MONIES,   OR OTHER SPECIAL,
     INCIDENTAL   OR  CONSEQUENTIAL DAMAGES   ARISING   OUT  OF  THE USE OR
     INABILITY TO USE (INCLUDING BUT NOT LIMITED TO LOSS OF DATA OR DATA OR
     ITS ANALYSIS BEING  RENDERED INACCURATE OR  LOSSES SUSTAINED  BY THIRD
     PARTIES) THE PROGRAM.
     
     (Above NO WARRANTY modified from the GNU NO WARRANTY statement.)

*/

/* normal.f -- translated by f2c (version 19940714). */

#include <stdio.h>

typedef long int integer;
typedef char *address;
typedef short int shortint;
typedef float real;
typedef double doublereal;
typedef struct { real r, i; } complex;
typedef struct { doublereal r, i; } doublecomplex;
typedef long int logical;
typedef short int shortlogical;
typedef char logical1;
typedef char integer1;
#define TRUE_ (1)
#define FALSE_ (0)

#define abs(x) ((x) >= 0 ? (x) : -(x))
#define dabs(x) (doublereal)abs(x)
#define min(a,b) ((a) <= (b) ? (a) : (b))
#define max(a,b) ((a) >= (b) ? (a) : (b))
#define dmin(a,b) (doublereal)min(a,b)
#define dmax(a,b) (doublereal)max(a,b)


/* Table of constant values */

static integer c__5 = 5;
static integer c__4 = 4;
static integer c__8 = 8;
static integer c__12 = 12;

/* Subroutine */ int cdfnor_(which, p, x, mean, sd, status, bound)
integer *which;
real *p, *x, *mean, *sd;
integer *status;
real *bound;
{
    static real z;
    extern doublereal cumnor_(), invnor_();

/* ********************************************************************** 
*/

/*      SUBROUTINE CDFNOR( WHICH, P, X, MEAN, SD, STATUS, BOUND ) */
/*               Cumulative Distribution Function */
/*               NORmal distribution */


/*                              Function */


/*     Calculates any one parameter of the normal */
/*     distribution given values for the others. */


/*                              Arguments */


/*     WHICH  --> Integer indicating  which of the  next  parameter */
/*     values is to be calculated using values  of the others. */
/*     Legal range: 1..4 */
/*                    INTEGER WHICH */

/*     P <--> The integral from -infinity to X of the normal density. */
/*            Input range: [1E-7,1-1E-7]. */
/*                    REAL P */

/*     X < --> Upper limit of integration of the normal-density. */
/*             Input range: ( -infinity, +infinity) */
/*                    REAL X */

/*     MEAN <--> The mean of the normal density. */
/*               Input range: (-infinity, +infinity) */
/*                    REAL MEAN */

/*     SD <--> Standard Deviation of the normal density. */
/*             Input range: (0, +infinity). */
/*                    REAL SD */

/*     STATUS <-- 0 if calculation completed correctly */
/*               -I if input parameter number I is out of range */
/*                1 if answer appears to be lower than lowest */
/*                  search bound */
/*                2 if answer appears to be higher than greatest */
/*                  search bound */
/*                    INTEGER STATUS */

/*     BOUND <-- Undefined if STATUS is 0 */

/*               Bound exceeded by parameter number I if STATUS */
/*               is negative. */

/*               Lower search bound if STATUS is 1. */

/*               Upper search bound if STATUS is 2. */


/*                              Method */


/*     The rational functions from pages  90-95  of Kennedy and Gentle, */
/*     Statistical  Computing,  Marcel  Dekker, NY,  1980 are  used  to */
/*     compute  both  the  cumulative  and   inverse  standard  normal. */
/*     Therefore no searches are necessary for any parameter. */

/*     For X < -15, the asymptotic expansion for the normal is used. */
/*     This is formula 26.2.12 of Abramowitz and Stegun. */


/*                              Note */


/*      The normal density is proportional to */
/*      exp( - 0.5 * (( X - MEAN)/SD)**2) */


/* ********************************************************************** 
*/
/*     .. Parameters .. */
/*     .. */
/*     .. Scalar Arguments .. */
/*     .. */
/*     .. Local Scalars .. */
/*     .. */
/*     .. External Functions .. */
/*     .. */
/*     .. Executable Statements .. */

/*     Check arguments */

    *status = 0;
    if (! (*which < 1 || *which > 4)) {
	goto L10;
    }
    if (*which < 1) {
	*bound = (float)1.;
    } else {
	*bound = (float)4.;
    }
    *status = -1;
    return 0;
L10:
    if (*which == 1) {
	goto L30;
    }

/*     P */

    if (! (*p < (float)1e-7 || *p > (float).99999990000000005)) {
	goto L20;
    }
    if (*p < (float)1e-7) {
	*bound = (float)1e-7;
    } else {
	*bound = (float).99999990000000005;
    }
    *status = -2;
    return 0;
L20:
L30:
    if (*which == 4) {
	goto L50;
    }

/*     SD */

    if (! (*sd <= (float)0.)) {
	goto L40;
    }
    *bound = (float)0.;
    *status = -5;
    return 0;
L40:

/*     Calculate ANSWERS */

L50:
    if (1 == *which) {

/*     Computing P */

	z = (*x - *mean) / *sd;
	*p = cumnor_(&z);
    } else if (2 == *which) {

/*     Computing X */

	z = invnor_(p);
	*x = *sd * z + *mean;
    } else if (3 == *which) {

/*     Computing the MEAN */

	z = invnor_(p);
	*mean = *x - *sd * z;
    } else if (4 == *which) {

/*     Computing SD */

	z = invnor_(p);
	*sd = (*x - *mean) / z;
    }
    return 0;
} /* cdfnor_ */

doublereal cumnor_(x)
real *x;
{
    /* System generated locals */
    real ret_val;
    doublereal d__1;

    /* Local variables */
    extern doublereal dumnor_();


/* ********************************************************************** 
*/

/*     REAL FUNCTION CUMNOR(X) */


/*                              Function */


/*     Computes the cumulative  of    the  normal   distribution,   i.e., 
*/
/*     the integral from -infinity to x of */
/*          (1/sqrt(2*pi)) exp(-u*u/2) du */


/*                              Method */


/*     Calls the double precision version of this routine DUMNOR. */


/*                              Arguments */


/*     X --> Value at which cumulative normal is evaluated */
/*                    REAL X */

/* ********************************************************************** 
*/
/*     .. */
/*     .. Scalar Arguments .. */
/*     .. */
/*     .. External Functions .. */
/*     .. */
/*     .. Executable Statements .. */
    d__1 = (doublereal) (*x);
    ret_val = dumnor_(&d__1);
    return ret_val;
} /* cumnor_ */

doublereal invnor_(p)
real *p;
{
    /* Initialized data */

    static doublereal xnum[5] = { -.322232431088,-1.,-.342242088547,
	    -.0204231210245,-4.53642210148e-5 };
    static doublereal xden[5] = { .099348462606,.588581570495,.531103462366,
	    .10353775285,.0038560700634 };

    /* System generated locals */
    real ret_val;

    /* Builtin functions */
    double log(), sqrt();

    /* Local variables */
    static doublereal sign, y, z;
    extern doublereal devlpl_();


/* ********************************************************************** 
*/

/*     REAL FUNCTION INVNOR(P) */
/*                    NORmal distribution INVerse */


/*                              Function */


/*     Returns X  such that CUMNOR(X)  =   P,  i.e., the  integral from - 
*/
/*     infinity to X of (1/SQRT(2*PI)) EXP(-U*U) dU is P */


/*                              Arguments */


/*     P --> The probability whose normal deviate is sought. */
/*                    P is REAL */


/*                              Method */


/*     The  rational   function   on  page 95    of Kennedy  and  Gentle, 
*/
/*     Statistical Computing, Marcel Dekker, NY , 1980. */


/*                              Note */


/*     If P .lt. 1.0e-20 then INVNOR returns -10.0. */
/*     If P .ge. 1.0 then INVNOR returns 10.0. */

/* ********************************************************************** 
*/

/*     .. Scalar Arguments .. */
/*     .. */
/*     .. Local Scalars .. */
/*     .. */
/*     .. Local Arrays .. */
/*     .. */
/*     .. External Functions .. */
/*     .. */
/*     .. Intrinsic Functions .. */
/*     .. */
/*     .. Data statements .. */
/*     .. */
/*     .. Executable Statements .. */
    if (! (*p < (float)1e-20)) {
	goto L10;
    }
    ret_val = (float)-10.;
    return ret_val;
L10:
    if (! (*p >= (float)1.)) {
	goto L20;
    }
    ret_val = (float)10.;
    return ret_val;
L20:
    if (! (*p <= .5)) {
	goto L30;
    }
    sign = -1.;
    z = *p;
    goto L40;
L30:
    sign = 1.;
    z = 1. - (doublereal) (*p);
L40:
    y = sqrt(log(z) * -2.);
    ret_val = y + devlpl_(xnum, &c__5, &y) / devlpl_(xden, &c__5, &y);
    ret_val = sign * ret_val;
    return ret_val;
} /* invnor_ */

doublereal dumnor_(x)
doublereal *x;
{
    /* Initialized data */

    static doublereal xden2[8] = { 300.4592609569832933,790.9509253278980272,
	    931.3540948506096211,638.9802644656311665,277.5854447439876434,
	    77.00015293522947295,12.78272731962942351,1. };
    static doublereal xnum3[5] = { -.00299610707703542174,
	    -.0494730910623250734,-.22695659353968693,-.278661308609647788,
	    -.0223192459734184686 };
    static doublereal xden3[5] = { .0106209230528467918,.191308926107829841,
	    1.05167510706793207,1.98733201817135256,1. };
    static doublereal pim12 = .5641895835477562869480795;
    static doublereal sqrt2 = 1.4142135623730950488;
    static doublereal xnum1[4] = { 242.66795523053175,21.979261618294152,
	    6.9963834886191355,-.035609843701815385 };
    static doublereal xden1[4] = { 215.0588758698612,91.164905404514901,
	    15.082797630407787,1. };
    static doublereal xnum2[8] = { 300.4592610201616005,451.9189537118729422,
	    339.320816734343687,152.9892850469404039,43.1622272220567353,
	    7.211758250883093659,.5641955174789739711,
	    -1.368648573827167067e-7 };

    /* System generated locals */
    doublereal ret_val, d__1;

    /* Builtin functions */
    double exp();

    /* Local variables */
    static doublereal derf, derfc, z, z2;
    extern doublereal dlanor_();
    static logical qdirct;
    extern doublereal devlpl_();
    static doublereal zm2;

/* ********************************************************************** 
*/

/*     DOUBLE PRECISION FUNCTION DUMNOR(X) */


/*                              Function */


/*     Computes the cumulative  of    the  normal   distribution,   i.e., 
*/
/*     the integral from -infinity to x of */
/*          (1/sqrt(2*pi)) exp(-u*u/2) du */


/*                              Method */


/*     The    rational function  approximation   from pages   90  - 92 of 
*/
/*     Kennedy  and Gentle,  Statistical  Computing,  Marcel  Dekker,  NY 
*/
/*     1980. */


/*                              Arguments */


/*     X --> Argument at which cumulative normal is evaluated */
/*                    DOUBLE PRECISION X */

/* ********************************************************************** 
*/


/*     PIM12 IS PI**(-1/2) */
/*     SQRT2 IS SQRT(2) */



/*     .. Scalar Arguments .. */
/*     .. */
/*     .. Local Scalars .. */
/*     .. */
/*     .. Local Arrays .. */
/*     .. */
/*     .. External Functions .. */
/*     .. */
/*     .. Intrinsic Functions .. */
/*     .. */
/*     .. Data statements .. */
/*     .. */
/*     .. Executable Statements .. */
    if (! (abs(*x) < (float)1e-30)) {
	goto L10;
    }
    ret_val = (float).5;
    return ret_val;
    goto L50;
L10:
    if (! (*x <= (float)-38.)) {
	goto L20;
    }
    ret_val = (float)0.;
    return ret_val;
    goto L50;
L20:
    if (! (*x <= (float)-15.)) {
	goto L30;
    }
    ret_val = exp(dlanor_(x));
    return ret_val;
    goto L50;
L30:
    if (! (*x > (float)6.)) {
	goto L40;
    }
    ret_val = (float)1.;
    return ret_val;
    goto L50;
L40:
L50:
    z = (d__1 = *x / sqrt2, abs(d__1));
    z2 = z * z;
    zm2 = 1. / z2;
    if (z < .5) {
	derf = z * devlpl_(xnum1, &c__4, &z2) / devlpl_(xden1, &c__4, &z2);
	qdirct = TRUE_;
    } else if (z < 4.) {
	derfc = exp(-z2) * devlpl_(xnum2, &c__8, &z) / devlpl_(xden2, &c__8, &
		z);
	qdirct = FALSE_;
    } else {
	derfc = exp(-z2) / z * (pim12 + zm2 * devlpl_(xnum3, &c__5, &zm2) / 
		devlpl_(xden3, &c__5, &zm2));
	qdirct = FALSE_;
    }
    if (! (*x >= (float)0.)) {
	goto L60;
    }
    if (! qdirct) {
	derf = 1. - derfc;
    }
    ret_val = (derf + 1.) / 2.;
    goto L70;
L60:
    if (qdirct) {
	derfc = 1. - derf;
    }
    ret_val = derfc / 2.;
L70:
    return ret_val;
} /* dumnor_ */

doublereal devlpl_(a, n, x)
doublereal *a;
integer *n;
doublereal *x;
{
    /* System generated locals */
    doublereal ret_val;

    /* Local variables */
    static doublereal term;
    static integer i;


/* ********************************************************************** 
*/

/*     DOUBLE PRECISION FUNCTION DEVLPL(A,N,X) */
/*              Double precision EVALuate a PoLynomial at X */


/*                              Function */


/*     returns */
/*          A(1) + A(2)*X + ... + A(N)*X**(N-1) */


/*                              Arguments */


/*     A --> Array of coefficients of the polynomial. */
/*                                        A is DOUBLE PRECISION(N) */

/*     N --> Length of A, also degree of polynomial - 1. */
/*                                        N is INTEGER */

/*     X --> Point at which the polynomial is to be evaluated. */
/*                                        X is DOUBLE PRECISION */

/* ********************************************************************** 
*/

/*     .. Scalar Arguments .. */
/*     .. */
/*     .. Array Arguments .. */
/*     .. */
/*     .. Local Scalars .. */
/*     .. */
/*     .. Executable Statements .. */
    /* Parameter adjustments */
    --a;

    /* Function Body */
    term = a[*n];
    for (i = *n - 1; i >= 1; --i) {
	term = a[i] + term * *x;
/* L10: */
    }
    ret_val = term;
    return ret_val;
} /* devlpl_ */

doublereal dlanor_(x)
doublereal *x;
{
    /* Initialized data */

    static doublereal coef[12] = { -1.,3.,-15.,105.,-945.,10395.,-135135.,
	    2027025.,-34459425.,654729075.,-13749310575.,316234143225. };

    /* System generated locals */
    doublereal ret_val, d__1;

    /* Builtin functions */
    double log();

    /* Local variables */
    extern doublereal dln1px_();
    static doublereal correc, xx;
    extern doublereal devlpl_();
    static doublereal approx, xx2;

/* ********************************************************************** 
*/

/*      DOUBLE PRECISION FUNCTION DLANOR( X ) */
/*           Double precision Logarith of the Asymptotic Normal */


/*                              Function */


/*      Computes the logarithm of the cumulative normal distribution */
/*      from abs( x ) to infinity for abs( x ) >= 5. */


/*                              Arguments */


/*      X --> Value at which cumulative normal to be evaluated */
/*                     DOUBLE PRECISION X */


/*                              Method */


/*      23 term expansion of formula 26.2.12 of Abramowitz and Stegun. */
/*      The relative error at X = 5 is about 0.5E-5. */


/*                              Note */


/*      ABS(X) must be >= 5 else there is an error stop. */

/* ********************************************************************** 
*/
/*     .. Parameters .. */
/*     .. */
/*     .. Scalar Arguments .. */
/*     .. */
/*     .. Local Scalars .. */
/*     .. */
/*     .. Local Arrays .. */
/*     .. */
/*     .. External Functions .. */
/*     .. */
/*     .. Intrinsic Functions .. */
/*     .. */
/*     .. Data statements .. */
/*     .. */
/*     .. Executable Statements .. */
    xx = abs(*x);
    if (xx < 5.) {
	fprintf(stderr," Argument too small in DLANOR");
	exit(1);
    }
    approx = -.91893853320467274177 - xx * (float).5 * xx - log(xx);
    xx2 = xx * xx;
    d__1 = 1. / xx2;
    correc = devlpl_(coef, &c__12, &d__1) / xx2;
    correc = dln1px_(&correc);
    ret_val = approx + correc;
    return ret_val;
} /* dlanor_ */

doublereal dln1px_(a)
doublereal *a;
{
    /* Initialized data */

    static doublereal p1 = -1.29418923021993;
    static doublereal p2 = .405303492862024;
    static doublereal p3 = -.0178874546012214;
    static doublereal q1 = -1.62752256355323;
    static doublereal q2 = .747811014037616;
    static doublereal q3 = -.0845104217945565;

    /* System generated locals */
    doublereal ret_val;

    /* Builtin functions */
    double log();

    /* Local variables */
    static doublereal t, w, x, t2;

/* ********************************************************************** 
*/

/*     DOUBLE PRECISION FUNCTION DLN1PX(X) */
/*               Double precision LN(1+X) */


/*                              Function */


/*     Returns ln(1+x) */
/*     Note that the obvious code of */
/*               LOG(1.0+X) */
/*     won't work for small X because 1.0+X loses accuracy */


/*                              Arguments */


/*     X --> Value for which ln(1-x) is desired. */
/*                                        X is DOUBLE PRECISION */


/*                              Method */


/*     Renames ALNREL from: */
/*     DiDinato, A. R. and Morris,  A.   H.  Algorithm 708: Significant */
/*     Digit Computation of the Incomplete  Beta  Function Ratios.  ACM */
/*     Trans. Math.  Softw. 18 (1993), 360-373. */

/* ********************************************************************** 
*/
/* -----------------------------------------------------------------------
 */
/*            EVALUATION OF THE FUNCTION LN(1 + A) */
/* -----------------------------------------------------------------------
 */
/*     .. Scalar Arguments .. */
/*     .. */
/*     .. Local Scalars .. */
/*     .. */
/*     .. Intrinsic Functions .. */
/*     .. */
/*     .. Data statements .. */
/*     .. */
/*     .. Executable Statements .. */
/* -------------------------- */
    if (abs(*a) > .375) {
	goto L10;
    }
    t = *a / (*a + 2.);
    t2 = t * t;
    w = (((p3 * t2 + p2) * t2 + p1) * t2 + 1.) / (((q3 * t2 + q2) * t2 + q1) *
	     t2 + 1.);
    ret_val = t * 2. * w;
    return ret_val;

L10:
    x = *a + 1.;
    ret_val = log(x);
    return ret_val;
} /* dln1px_ */

